# $Id: Stub.pm 38114 2007-05-04 19:36:12Z wsnyder $
# DESCRIPTION: Example perl module that just "works"
#
# Copyright 2007-2007 by Wilson Snyder <wsnyder@wsnyder.org>.  This
# program is free software; you can redistribute it and/or modify it under
# the terms of either the GNU Lesser General Public License or the Perl
# Artistic License.
######################################################################

package Stub;

print "ok: in package Stub\n";

######################################################################
1;
